"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import random
import string


def random_digits(n=8):
    return ''.join([random.choice(string.digits) for _ in range(n)])


def random_tld(name='testtld'):
    return f'{name}{random_digits()}'


def random_tsigkey_name(name='testtsig'):
    return f'{name}{random_digits()}'


def random_tsigkey_secret(name='test-secret'):
    return f'{name}-{random_digits(254 - len(name))}'


def random_zone_name(name='testdomain', tld='com'):
    return f'{name}{random_digits()}.{tld}.'


def random_a_recordset_name(zone_name, recordset_name='testrecord'):
    return f'{recordset_name}{random_digits()}.{zone_name}'


def random_blacklist(name='testblacklist'):
    return f'{name}{random_digits()}'


def random_zone_file(name='testzoneimport'):
    return (
        '$ORIGIN {0}{1}.com.\n'
        '$TTL 300\n'
        '{0}{1}.com. 300 IN SOA ns.{0}{1}.com. '
        'nsadmin.{0}{1}.com. 42 42 42 42 42\n'
        '{0}{1}.com. 300 IN NS ns.{0}{1}.com.\n'
        '{0}{1}.com. 300 IN MX 10 mail.{0}{1}.com.\n'
        'ns.{0}{1}.com. 300 IN A 10.0.0.1\n'
        'mail.{0}{1}.com. 300 IN A 10.0.0.2\n'.format(
            name, random_digits()
        )
    )



# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import fixtures
from keystoneauth1 import session as keystone_session
from oslo_serialization import jsonutils
from oslotest import base as test
from requests_mock.contrib import fixture as req_fixture
from urllib import parse as urlparse

from designateclient import client
from designateclient.utils import AdapterWithTimeout

_TRUE_VALUES = ('True', 'true', '1', 'yes')


class TestCase(test.BaseTestCase):

    """Test case base class for all unit tests."""

    def setUp(self):
        """Run before each test method to initialize test environment."""

        super().setUp()
        test_timeout = os.environ.get('OS_TEST_TIMEOUT', 0)
        try:
            test_timeout = int(test_timeout)
        except ValueError:
            # If timeout value is invalid do not set a timeout.
            test_timeout = 0
        if test_timeout > 0:
            self.useFixture(fixtures.Timeout(test_timeout, gentle=True))

        self.useFixture(fixtures.NestedTempfile())
        self.useFixture(fixtures.TempHomeDir())

        if os.environ.get('OS_STDOUT_CAPTURE') in _TRUE_VALUES:
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if os.environ.get('OS_STDERR_CAPTURE') in _TRUE_VALUES:
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))

        self.log_fixture = self.useFixture(fixtures.FakeLogger())


class APITestCase(TestCase):
    """Test case base class for all unit tests."""

    TEST_URL = "http://127.0.0.1:9001/"
    VERSION = None

    def setUp(self):
        """Run before each test method to initialize test environment."""
        super(TestCase, self).setUp()
        self.log_fixture = self.useFixture(fixtures.FakeLogger())

        self.requests = self.useFixture(req_fixture.Fixture())
        self.client = self.get_client()

    def get_base(self, base_url=None):
        if not base_url:
            base_url = f'{self.TEST_URL}v{self.VERSION}'
        return base_url

    def stub_url(self, method, parts=None, base_url=None, json=None, **kwargs):
        base_url = self.get_base(base_url)

        if json:
            kwargs['text'] = jsonutils.dumps(json)
            headers = kwargs.setdefault('headers', {})
            headers['Content-Type'] = 'application/json'

        if parts:
            url = '/'.join([p.strip('/') for p in [base_url] + parts])
        else:
            url = base_url

        url = url.replace("/?", "?")
        self.requests.register_uri(method, url, **kwargs)

    def get_client(self, version=None, session=None):
        version = version or self.VERSION
        session = session or keystone_session.Session()
        adapted = AdapterWithTimeout(
            session=session, endpoint_override=self.get_base())
        return client.Client(version, session=adapted)

    def assertRequestBodyIs(self, body=None, json=None):
        last_request_body = self.requests.last_request.body
        if json:
            val = jsonutils.loads(last_request_body)
            self.assertEqual(json, val)
        elif body:
            self.assertEqual(body, last_request_body)

    def assertQueryStringIs(self, qs=''):
        """Verify the QueryString matches what is expected.

        The qs parameter should be of the format \'foo=bar&abc=xyz\'
        """
        expected = urlparse.parse_qs(qs, keep_blank_values=True)
        parts = urlparse.urlparse(self.requests.last_request.url)
        querystring = urlparse.parse_qs(parts.query, keep_blank_values=True)
        self.assertEqual(expected, querystring)

    def assertQueryStringContains(self, **kwargs):
        """Verify the query string contains the expected parameters.

        This method is used to verify that the query string for the most recent
        request made contains all the parameters provided as ``kwargs``, and
        that the value of each parameter contains the value for the kwarg. If
        the value for the kwarg is an empty string (''), then all that's
        verified is that the parameter is present.

        """
        parts = urlparse.urlparse(self.requests.last_request.url)
        qs = urlparse.parse_qs(parts.query, keep_blank_values=True)

        for k, v in kwargs.items():
            self.assertIn(k, qs)
            self.assertIn(v, qs[k])

    def assertRequestHeaderEqual(self, name, val):
        """Verify that the last request made contains a header and its value

        The request must have already been made.
        """
        headers = self.requests.last_request.headers
        self.assertEqual(val, headers.get(name))



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Base(Exception):
    def __init__(self, message=None):
        if not message:
            message = self.__class__.__name__
        super().__init__(message)


class UnsupportedVersion(Base):
    pass


class ResourceNotFound(Base):
    pass


class NoUniqueMatch(Base):
    pass


class RemoteError(Base):
    def __init__(self, message=None, code=None, type=None, errors=None,
                 request_id=None, **ignore):
        err_message = self._get_error_message(message, type, errors)
        self.message = err_message
        self.code = code
        self.type = type
        self.errors = errors
        self.request_id = request_id

        super().__init__(err_message)

    def _get_error_message(self, _message, _type, _errors):
        # Try to get a useful error msg if 'message' has nothing
        if not _message:
            if _errors and 'errors' in _errors:
                err_msg = list()
                for err in _errors['errors']:
                    if 'message' in err:
                        err_msg.append(err['message'])
                _message = '. '.join(err_msg)
            elif _type:
                _message = str(_type)
        return _message


class Unknown(RemoteError):
    pass


class BadRequest(RemoteError):
    pass


class Forbidden(RemoteError):
    pass


class Conflict(RemoteError):
    pass


class NotFound(RemoteError):
    pass


class OverQuota(RemoteError):
    pass



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# Copyright (c) 2012, Cloudscaling
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import re

from hacking import core

# D701: Default parameter value is a mutable type
# D702: Log messages require translation
# D703: Found use of _() without explicit import of _!
# D704: Found import of %s. This oslo library has been graduated!
# D705: timeutils.utcnow() must be used instead of datetime.%s()
# D706: Don't translate debug level logs
# D707: basestring is not Python3-compatible, use str instead.
# D708: Do not use xrange. Use range for large loops.
# D709: LOG.audit is deprecated, please use LOG.info!
# D710: LOG.warn() is not allowed. Use LOG.warning()
# D711: Don't use backslashes for line continuation.

UNDERSCORE_IMPORT_FILES = []


mutable_default_argument_check = re.compile(
    r"^\s*def .+\((.+=\{\}|.+=\[\])")
string_translation = re.compile(r"[^_]*_\(\s*('|\")")
translated_log = re.compile(
    r"(.)*LOG\.(audit|error|info|warn|warning|critical|exception)"
    r"\(\s*_\(\s*('|\")")
underscore_import_check = re.compile(r"(.)*import _(.)*")
# We need this for cases where they have created their own _ function.
custom_underscore_check = re.compile(r"(.)*_\s*=\s*(.)*")
graduated_oslo_libraries_import_re = re.compile(
    r"^\s*(?:import|from) designate\.openstack\.common\.?.*?"
    r"(gettextutils|rpc)"
    r".*?")
no_line_continuation_backslash_re = re.compile(r'.*(\\)\n')


@core.flake8ext
def mutable_default_arguments(logical_line, filename, noqa):
    if noqa:
        return

    if mutable_default_argument_check.match(logical_line):
        yield (0, "D701: Default parameter value is a mutable type")


@core.flake8ext
def no_translate_debug_logs(logical_line, filename):
    """Check for 'LOG.debug(_('
    As per our translation policy,
    https://wiki.openstack.org/wiki/LoggingStandards#Log_Translation
    we shouldn't translate debug level logs.
    * This check assumes that 'LOG' is a logger.
    * Use filename so we can start enforcing this in specific folders instead
      of needing to do so all at once.
    N319
    """
    if logical_line.startswith("LOG.debug(_("):
        yield (0, "D706: Don't translate debug level logs")


@core.flake8ext
def check_explicit_underscore_import(logical_line, filename):
    """Check for explicit import of the _ function

    We need to ensure that any files that are using the _() function
    to translate logs are explicitly importing the _ function.  We
    can't trust unit test to catch whether the import has been
    added so we need to check for it here.
    """
    # Build a list of the files that have _ imported.  No further
    # checking needed once it is found.
    if filename in UNDERSCORE_IMPORT_FILES:
        pass
    elif (underscore_import_check.match(logical_line) or
          custom_underscore_check.match(logical_line)):
        UNDERSCORE_IMPORT_FILES.append(filename)
    elif (translated_log.match(logical_line) or
          string_translation.match(logical_line)):
        yield (0, "D703: Found use of _() without explicit import of _!")


@core.flake8ext
def no_import_graduated_oslo_libraries(logical_line, filename):
    """Check that we don't continue to use o.c. oslo libraries after graduation

    After a library graduates from oslo-incubator, as we make the switch, we
    should ensure we don't continue to use the oslo-incubator versions.

    In many cases, it's not possible to immediately remove the code from the
    openstack/common folder due to dependency issues.
    """
    # We can't modify oslo-incubator code, so ignore it here.
    if "designate/openstack/common" in filename:
        return

    matches = graduated_oslo_libraries_import_re.match(logical_line)
    if matches:
        yield (0, "D704: Found import of %s. This oslo library has been "
                  "graduated!" % matches.group(1))


@core.flake8ext
def use_timeutils_utcnow(logical_line, filename):
    # tools are OK to use the standard datetime module
    if "/tools/" in filename:
        return

    msg = "D705: timeutils.utcnow() must be used instead of datetime.%s()"

    datetime_funcs = ['now', 'utcnow']
    for f in datetime_funcs:
        pos = logical_line.find('datetime.%s' % f)
        if pos != -1:
            yield (pos, msg % f)


@core.flake8ext
def check_no_basestring(logical_line):
    if re.search(r"\bbasestring\b", logical_line):
        msg = ("D707: basestring is not Python3-compatible, use "
               "str instead.")
        yield (0, msg)


@core.flake8ext
def check_python3_xrange(logical_line):
    if re.search(r"\bxrange\s*\(", logical_line):
        yield (0, "D708: Do not use xrange. Use range for "
                  "large loops.")


@core.flake8ext
def check_no_log_audit(logical_line):
    """Ensure that we are not using LOG.audit messages
    Plans are in place going forward as discussed in the following
    spec (https://review.opendev.org/#/c/132552/) to take out
    LOG.audit messages. Given that audit was a concept invented
    for OpenStack we can enforce not using it.
    """
    if "LOG.audit(" in logical_line:
        yield (0, "D709: LOG.audit is deprecated, please use LOG.info!")


@core.flake8ext
def check_no_log_warn(logical_line):
    """Disallow 'LOG.warn('

    D710
    """
    if logical_line.startswith('LOG.warn('):
        yield (0, "D710:Use LOG.warning() rather than LOG.warn()")


@core.flake8ext
def check_line_continuation_no_backslash(logical_line, tokens):
    """D711 - Don't use backslashes for line continuation.

    :param logical_line: The logical line to check. Not actually used.
    :param tokens: List of tokens to check.
    :returns: None if the tokens don't contain any issues, otherwise a tuple
              is yielded that contains the offending index in the logical
              line and a message describe the check validation failure.
    """
    backslash = None
    for token_type, text, start, end, orig_line in tokens:
        m = no_line_continuation_backslash_re.match(orig_line)
        if m:
            backslash = (start[0], m.start(1))
            break

    if backslash is not None:
        msg = 'D711 Backslash line continuations not allowed'
        yield backslash, msg



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from stevedore import extension
from urllib import parse

from oslo_serialization import jsonutils

from designateclient import exceptions


class Controller(metaclass=abc.ABCMeta):

    def __init__(self, client):
        self.client = client

    def build_url(self, url, criterion=None, marker=None, limit=None):
        params = criterion or {}

        if marker is not None:
            params['marker'] = marker
        if limit is not None:
            params['limit'] = limit

        q = parse.urlencode(params) if params else ''
        return '{url}{params}'.format(
            url=url,
            params=f'?{q}'
        )

    def _serialize(self, kwargs):
        headers = kwargs.get('headers')
        content_type = headers.get('Content-Type') if headers else None

        if 'data' in kwargs and content_type in {None, 'application/json'}:
            kwargs['data'] = jsonutils.dumps(kwargs['data'])

    def _post(self, url, response_key=None, **kwargs):
        self._serialize(kwargs)

        resp, body = self.client.session.post(url, **kwargs)
        if response_key is not None:
            return body[response_key]
        return body

    def _get(self, url, response_key=None):
        resp, body = self.client.session.get(url)
        if response_key is not None:
            return body[response_key]
        return body

    def _patch(self, url, response_key=None, **kwargs):
        self._serialize(kwargs)

        resp, body = self.client.session.patch(url, **kwargs)
        if response_key is not None:
            return body[response_key]
        return body

    def _put(self, url, response_key=None, **kwargs):
        self._serialize(kwargs)

        resp, body = self.client.session.put(url, **kwargs)
        if response_key is not None:
            return body[response_key]
        return body

    def _delete(self, url, response_key=None, **kwargs):
        resp, body = self.client.session.delete(url, **kwargs)
        if response_key is not None:
            return body[response_key]
        return body


class CrudController(Controller, metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def list(self, *args, **kw):
        """
        List a resource
        """

    @abc.abstractmethod
    def get(self, *args, **kw):
        """
        Get a resource
        """

    @abc.abstractmethod
    def create(self, *args, **kw):
        """
        Create a resource
        """

    @abc.abstractmethod
    def update(self, *args, **kw):
        """
        Update a resource
        """

    @abc.abstractmethod
    def delete(self, *args, **kw):
        """
        Delete a resource
            """


def get_versions():
    mgr = extension.ExtensionManager('designateclient.versions')
    return {ep.name: ep.plugin for ep in mgr.extensions}


def Client(version, *args, **kwargs):  # noqa
    versions = get_versions()
    if version not in versions:
        msg = 'Version {} is not supported, use one of ({})'.format(
            version, list(versions.keys())
        )
        raise exceptions.UnsupportedVersion(msg)
    return versions[version](*args, **kwargs)



"""
Copyright 2016 Rackspace

Author: Rahman Syed <rahman.syed@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from designateclient.functionaltests.base import BaseDesignateTest
from designateclient.functionaltests.datagen import random_zone_file
from designateclient.functionaltests.v2.fixtures import ImportFixture


class TestZoneImport(BaseDesignateTest):

    def setUp(self):
        super().setUp()
        self.ensure_tld_exists('com')
        self.zone_file_contents = random_zone_file()

    def test_list_zone_imports(self):
        zone_import = self.useFixture(ImportFixture(
            zone_file_contents=self.zone_file_contents
        )).zone_import

        zone_imports = self.clients.zone_import_list()
        self.assertGreater(len(zone_imports), 0)
        self.assertTrue(self._is_entity_in_list(zone_import, zone_imports))

    def test_create_and_show_zone_import(self):
        zone_import = self.useFixture(ImportFixture(
            zone_file_contents=self.zone_file_contents
        )).zone_import

        fetched_import = self.clients.zone_import_show(zone_import.id)

        self.assertEqual(zone_import.created_at, fetched_import.created_at)
        self.assertEqual(zone_import.id, fetched_import.id)
        self.assertEqual(zone_import.project_id, fetched_import.project_id)

        # check both statuses to avoid a race condition, causing test failure.
        # we don't know when the import completes.
        self.assertIn(fetched_import.status, ['PENDING', 'COMPLETE'])

    def test_delete_zone_import(self):
        zone_import = self.useFixture(ImportFixture(
            zone_file_contents=self.zone_file_contents
        )).zone_import

        zone_imports = self.clients.zone_import_list()
        self.assertTrue(self._is_entity_in_list(zone_import, zone_imports))

        self.clients.zone_import_delete(zone_import.id)

        zone_imports = self.clients.zone_import_list()
        self.assertFalse(self._is_entity_in_list(zone_import, zone_imports))



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from designateclient.tests import v2

FIP_ID = '{}:{}'.format(str(uuid.uuid4()), "RegionOne")


class TestFloatingIP(v2.APIV2TestCase, v2.CrudMixin):
    def test_set(self):
        name = "foo.com."

        ref = {
            "ptrdname": name,
            "description": "foo"
        }

        parts = ["reverse", "floatingips", FIP_ID]
        self.stub_url("PATCH", parts=parts, json=ref)

        self.client.floatingips.set(FIP_ID, name, "foo")

    def test_list(self):
        ref = [
            {"ptrdname": "foo.com."}
        ]

        self.stub_url("GET", parts=["reverse", "floatingips"],
                      json={"floatingips": ref})

        self.client.floatingips.list()

    def test_get(self):
        ref = {
            "ptrdname": "foo.com."
        }

        parts = ["reverse", "floatingips", FIP_ID]
        self.stub_url("GET", parts=parts, json=ref)

        self.client.floatingips.get(FIP_ID)

    def test_unset(self):
        parts = ["reverse", "floatingips", FIP_ID]
        self.stub_url("PATCH", parts=parts, json={"ptdrname": None})

        self.client.floatingips.unset(FIP_ID)
        self.assertRequestBodyIs(None)



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from osc_lib.command import command

from designateclient import utils
from designateclient.v2.cli import common
from designateclient.v2 import utils as v2_utils


LOG = logging.getLogger(__name__)


def _format_status(status):
    status.pop('links', None)
    # Remove unneeded fields for display output formatting
    for k in ('capabilities', 'stats'):
        status[k] = '\n'.join(status[k]) if status[k] else '-'
    return status


class ListServiceStatusesCommand(command.Lister):
    """List service statuses"""

    columns = ['id', 'hostname', 'service_name', 'status', 'stats',
               'capabilities']

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('--hostname', help='Hostname', required=False)
        parser.add_argument('--service_name', help='Service Name',
                            required=False)
        parser.add_argument('--status', help='Status', required=False)
        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        cols = self.columns

        criterion = {}
        for i in ['hostname', 'service_name', 'status']:
            v = getattr(parsed_args, i)
            if v is not None:
                criterion[i] = v

        data = v2_utils.get_all(client.service_statuses.list,
                                criterion=criterion)

        for i, s in enumerate(data):
            data[i] = _format_status(s)

        return cols, (utils.get_item_properties(s, cols) for s in data)


class ShowServiceStatusCommand(command.ShowOne):
    """Show service status details"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('id', help='Service Status ID')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        data = client.service_statuses.get(parsed_args.id)

        _format_status(data)
        return zip(*sorted(data.items()))



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient.v2.base import V2Controller
from designateclient.v2 import utils as v2_utils


class ZoneController(V2Controller):
    def create(self, name, type_=None, email=None, description=None, ttl=None,
               masters=None, attributes=None):
        type_ = type_ or 'PRIMARY'

        data = {
            'name': name,
            'type': type_
        }

        if type_ == 'PRIMARY':
            if email:
                data['email'] = email

            if ttl is not None:
                data['ttl'] = ttl

        elif type_ == 'SECONDARY' and masters:
            data['masters'] = masters

        if description is not None:
            data['description'] = description

        if attributes is not None:
            data['attributes'] = attributes

        return self._post('/zones', data=data)

    def list(self, criterion=None, marker=None, limit=None):
        url = self.build_url('/zones', criterion, marker, limit)

        return self._get(url, response_key='zones')

    def get(self, zone):
        zone = v2_utils.resolve_by_name(self.list, zone)

        return self._get(f'/zones/{zone}')

    def update(self, zone, values):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = self.build_url(f'/zones/{zone}')

        return self._patch(url, data=values)

    def delete(self, zone, delete_shares=False):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = self.build_url(f'/zones/{zone}')

        if delete_shares:
            headers = {'X-Designate-Delete-Shares': 'true'}
            _resp, body = self.client.session.delete(url, headers=headers)
        else:
            _resp, body = self.client.session.delete(url)

        return body

    def abandon(self, zone):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = f'/zones/{zone}/tasks/abandon'

        self.client.session.post(url)

    def axfr(self, zone):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = f'/zones/{zone}/tasks/xfr'

        self.client.session.post(url)

    def pool_move(self, zone, values):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = self.build_url('/zones/%s/tasks/pool_move' % zone)

        return self._post(url, data=values)


class ZoneTransfersController(V2Controller):
    def create_request(self, zone, target_project_id, description=None):
        zone = v2_utils.resolve_by_name(self.client.zones.list, zone)

        data = {
            'target_project_id': target_project_id
        }

        if description is not None:
            data['description'] = description

        url = f'/zones/{zone}/tasks/transfer_requests'

        return self._post(url, data=data)

    def get_request(self, transfer_id):
        url = f'/zones/tasks/transfer_requests/{transfer_id}'
        return self._get(url)

    def list_requests(self):
        url = '/zones/tasks/transfer_requests'
        return self._get(url, response_key='transfer_requests')

    def update_request(self, transfer_id, values):
        url = f'/zones/tasks/transfer_requests/{transfer_id}'
        return self._patch(url, data=values)

    def delete_request(self, transfer_id):
        url = f'/zones/tasks/transfer_requests/{transfer_id}'
        self._delete(url)

    def accept_request(self, transfer_id, key):
        url = '/zones/tasks/transfer_accepts'

        data = {
            'key': key,
            'zone_transfer_request_id': transfer_id
        }
        return self._post(url, data=data)

    def get_accept(self, accept_id):
        url = f'/zones/tasks/transfer_accepts/{accept_id}'
        return self._get(url)

    def list_accepts(self):
        url = '/zones/tasks/transfer_accepts'
        return self._get(url, response_key='transfer_accepts')


class ZoneExportsController(V2Controller):
    def create(self, zone):
        zone_id = v2_utils.resolve_by_name(self.client.zones.list, zone)

        return self._post(f'/zones/{zone_id}/tasks/export')

    def get_export_record(self, zone_export_id):
        return self._get(f'/zones/tasks/exports/{zone_export_id}')

    def list(self):
        return self._get('/zones/tasks/exports')

    def delete(self, zone_export_id):
        return self._delete(f'/zones/tasks/exports/{zone_export_id}')

    def get_export(self, zone_export_id):
        return self._get(f'/zones/tasks/exports/{zone_export_id}/export',
                         headers={'Accept': 'text/dns'})


class ZoneImportsController(V2Controller):
    def create(self, zone_file_contents):
        return self._post('/zones/tasks/imports', data=zone_file_contents,
                          headers={'Content-Type': 'text/dns'})

    def get_import_record(self, zone_import_id):
        return self._get(f'/zones/tasks/imports/{zone_import_id}')

    def list(self):
        return self._get('/zones/tasks/imports')

    def delete(self, zone_import_id):
        return self._delete(f'/zones/tasks/imports/{zone_import_id}')


class ZoneShareController(V2Controller):
    def create(self, zone, target_project_id):
        zone_id = v2_utils.resolve_by_name(self.client.zones.list, zone)

        data = {'target_project_id': target_project_id}

        return self._post(f'/zones/{zone_id}/shares', data=data)

    def list(self, zone, criterion=None, marker=None, limit=None):
        zone_id = v2_utils.resolve_by_name(self.client.zones.list, zone)
        url = self.build_url(f'/zones/{zone_id}/shares',
                             criterion, marker, limit)

        return self._get(url, response_key='shared_zones')

    def delete(self, zone, shared_zone_id):
        zone_id = v2_utils.resolve_by_name(self.client.zones.list, zone)

        return self._delete(f'/zones/{zone_id}/shares/{shared_zone_id}')

    def get(self, zone, shared_zone_id):
        zone_id = v2_utils.resolve_by_name(self.client.zones.list, zone)

        return self._get(f'/zones/{zone_id}/shares/{shared_zone_id}')



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient.v2.base import V2Controller
from designateclient.v2 import utils as v2_utils


class TLDController(V2Controller):
    def create(self, name, description=None):
        data = {
            'name': name,
        }

        if description is not None:
            data['description'] = description

        return self._post('/tlds', data=data)

    def list(self, criterion=None, marker=None, limit=None):
        url = self.build_url('/tlds', criterion, marker, limit)

        return self._get(url, response_key='tlds')

    def get(self, tld):
        tld = v2_utils.resolve_by_name(self.list, tld)

        return self._get(f'/tlds/{tld}')

    def update(self, tld, values):
        tld = v2_utils.resolve_by_name(self.list, tld)

        return self._patch(f'/tlds/{tld}', data=values)

    def delete(self, tld):
        tld = v2_utils.resolve_by_name(self.list, tld)

        return self._delete(f'/tlds/{tld}')



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from osc_lib.command import command

from designateclient import utils
from designateclient.v2.cli import common
from designateclient.v2.utils import get_all


LOG = logging.getLogger(__name__)


def _format_floatingip(fip):
    # Remove unneeded fields for display output formatting
    fip.pop('links', None)


class ListFloatingIPCommand(command.Lister):
    """List floatingip ptr records"""

    columns = ['id', 'ptrdname', 'description', 'ttl']

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        cols = self.columns
        data = get_all(client.floatingips.list)
        return cols, (utils.get_item_properties(s, cols) for s in data)


class ShowFloatingIPCommand(command.ShowOne):
    """Show floatingip ptr record details"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('floatingip_id', help='Floating IP ID in format '
                            'region:floatingip_id')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        data = client.floatingips.get(parsed_args.floatingip_id)
        _format_floatingip(data)
        return zip(*sorted(data.items()))


class SetFloatingIPCommand(command.ShowOne):
    """Set floatingip ptr record"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('floatingip_id', help='Floating IP ID in format '
                            'region:floatingip_id')
        parser.add_argument('ptrdname', help='PTRD Name')

        description_group = parser.add_mutually_exclusive_group()
        description_group.add_argument('--description', help='Description')
        description_group.add_argument('--no-description', action='store_true')

        ttl_group = parser.add_mutually_exclusive_group()
        ttl_group.add_argument('--ttl', type=int, help='TTL')
        ttl_group.add_argument('--no-ttl', action='store_true')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        data = {}

        if parsed_args.no_description:
            data['description'] = None
        elif parsed_args.description:
            data['description'] = parsed_args.description

        if parsed_args.no_ttl:
            data['ttl'] = None
        elif parsed_args.ttl:
            data['ttl'] = parsed_args.ttl

        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        fip = client.floatingips.set(
            parsed_args.floatingip_id,
            parsed_args.ptrdname,
            parsed_args.description,
            parsed_args.ttl)

        _format_floatingip(fip)
        return zip(*sorted(fip.items()))


class UnsetFloatingIPCommand(command.Command):
    """Unset floatingip ptr record"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('floatingip_id', help='Floating IP ID in format '
                            'region:floatingip_id')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        client.floatingips.unset(parsed_args.floatingip_id)
        LOG.info('FloatingIP PTR %s was unset', parsed_args.floatingip_id)



"""
Copyright 2016 Rackspace

Author: Rahman Syed <rahman.syed@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from designateclient.functionaltests.base import BaseDesignateTest
from designateclient.functionaltests.datagen import random_zone_name
from designateclient.functionaltests.v2.fixtures import ExportFixture
from designateclient.functionaltests.v2.fixtures import ZoneFixture


class TestZoneExport(BaseDesignateTest):

    def setUp(self):
        super().setUp()
        self.ensure_tld_exists('com')
        fixture = self.useFixture(ZoneFixture(
            name=random_zone_name(),
            email='test@example.com',
        ))
        self.zone = fixture.zone

    def test_list_zone_exports(self):
        zone_export = self.useFixture(ExportFixture(
            zone=self.zone
        )).zone_export

        zone_exports = self.clients.zone_export_list()
        self.assertGreater(len(zone_exports), 0)
        self.assertTrue(self._is_entity_in_list(zone_export, zone_exports))

    def test_create_and_show_zone_export(self):
        zone_export = self.useFixture(ExportFixture(
            zone=self.zone
        )).zone_export

        fetched_export = self.clients.zone_export_show(zone_export.id)

        self.assertEqual(zone_export.created_at, fetched_export.created_at)
        self.assertEqual(zone_export.id, fetched_export.id)
        self.assertEqual(zone_export.message, fetched_export.message)
        self.assertEqual(zone_export.project_id, fetched_export.project_id)
        self.assertEqual(zone_export.zone_id, fetched_export.zone_id)

    def test_delete_zone_export(self):
        zone_export = self.useFixture(ExportFixture(
            zone=self.zone
        )).zone_export

        zone_exports = self.clients.zone_export_list()
        self.assertTrue(self._is_entity_in_list(zone_export, zone_exports))

        self.clients.zone_export_delete(zone_export.id)

        zone_exports = self.clients.zone_export_list()
        self.assertFalse(self._is_entity_in_list(zone_export, zone_exports))

    def test_show_export_file(self):
        zone_export = self.useFixture(ExportFixture(
            zone=self.zone
        )).zone_export

        fetched_export = self.clients.zone_export_showfile(zone_export.id)

        self.assertIn('$ORIGIN', fetched_export.data)
        self.assertIn('$TTL', fetched_export.data)
        self.assertIn('SOA', fetched_export.data)
        self.assertIn('NS', fetched_export.data)
        self.assertIn(self.zone.name, fetched_export.data)



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
# Author: Graham Hayes <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


def add_all_projects_option(parser):
    parser.add_argument(
        '--all-projects',
        default=False,
        action='store_true',
        help='Show results from all projects. Default: False'
    )


def add_edit_managed_option(parser):
    parser.add_argument(
        '--edit-managed',
        default=False,
        action='store_true',
        help='Edit resources marked as managed. Default: False'
    )


def add_hard_delete_option(parser):
    parser.add_argument(
        '--hard-delete',
        default=False,
        action='store_true',
        help='Delete zone along-with backend zone resources (i.e. files). '
             'Default: False'
    )


def add_sudo_project_id_option(parser):
    parser.add_argument(
        '--sudo-project-id',
        default=None,
        help='Project ID to impersonate for this command. Default: None'
    )


def add_all_common_options(parser):
    add_all_projects_option(parser)
    add_sudo_project_id_option(parser)


def set_all_projects(client, value):
    client.session.all_projects = value


def set_sudo_project_id(client, value):
    client.session.sudo_project_id = value


def set_edit_managed(client, value):
    client.session.edit_managed = value


def set_hard_delete(client, value):
    client.session.hard_delete = value


def set_all_common_headers(client, parsed_args):

    if (parsed_args.all_projects is not None and
            isinstance(parsed_args.all_projects, bool)):
        set_all_projects(client, parsed_args.all_projects)

    if (hasattr(parsed_args, 'edit_managed') and
            parsed_args.edit_managed is not None and
            isinstance(parsed_args.edit_managed, bool)):
        set_edit_managed(client, parsed_args.edit_managed)

    if (parsed_args.sudo_project_id is not None and
            isinstance(parsed_args.sudo_project_id, str)):
        set_sudo_project_id(client, parsed_args.sudo_project_id)

    if (hasattr(parsed_args, 'hard_delete') and
            parsed_args.hard_delete is not None and
            isinstance(parsed_args.hard_delete, bool)):
        set_hard_delete(client, parsed_args.hard_delete)



"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from tempest.lib.exceptions import CommandFailed

from designateclient.functionaltests.base import BaseDesignateTest
from designateclient.functionaltests.datagen import random_zone_name
from designateclient.functionaltests.v2.fixtures import ZoneFixture


class TestZone(BaseDesignateTest):

    def setUp(self):
        super().setUp()
        self.ensure_tld_exists('com')
        self.fixture = self.useFixture(ZoneFixture(
            name=random_zone_name(),
            email='test@example.com',
        ))
        self.zone = self.fixture.zone

    def test_zone_list(self):
        zones = self.clients.zone_list()
        self.assertGreater(len(zones), 0)

    def test_zone_create_and_show(self):
        zone = self.clients.zone_show(self.zone.id)
        self.assertTrue(hasattr(zone, 'action'))
        self.assertEqual(self.zone.created_at, zone.created_at)
        self.assertEqual(self.zone.description, zone.description)
        self.assertEqual(self.zone.email, zone.email)
        self.assertEqual(self.zone.id, zone.id)
        self.assertEqual(self.zone.masters, zone.masters)
        self.assertEqual(self.zone.name, zone.name)
        self.assertEqual(self.zone.pool_id, zone.pool_id)
        self.assertEqual(self.zone.project_id, zone.project_id)
        self.assertEqual(self.zone.serial, zone.serial)
        self.assertTrue(hasattr(zone, 'status'))
        self.assertEqual(self.zone.transferred_at, zone.transferred_at)
        self.assertEqual(self.zone.ttl, zone.ttl)
        self.assertEqual(self.zone.type, zone.type)
        self.assertEqual(self.zone.updated_at, zone.updated_at)
        self.assertEqual(self.zone.version, zone.version)

    def test_zone_delete(self):
        zone = self.clients.zone_delete(self.zone.id)
        self.assertEqual(zone.action, 'DELETE')
        self.assertEqual(zone.status, 'PENDING')

    def test_zone_set(self):
        ttl = int(self.zone.ttl) + 123
        email = f'updated{self.zone.email}'
        description = 'new description'

        zone = self.clients.zone_set(self.zone.id, ttl=ttl, email=email,
                                     description=description)
        self.assertEqual(ttl, int(zone.ttl))
        self.assertEqual(email, zone.email)
        self.assertEqual(description, zone.description)

    def test_invalid_option_on_zone_create(self):
        cmd = 'zone create {} --invalid "not a valid option"'.format(
            random_zone_name()
        )
        self.assertRaises(CommandFailed, self.clients.openstack, cmd)

    def test_invalid_zone_command(self):
        cmd = 'zone hopefullynotacommand'
        self.assertRaises(CommandFailed, self.clients.openstack, cmd)


class TestsPassingZoneFlags(BaseDesignateTest):

    def setUp(self):
        super().setUp()
        self.ensure_tld_exists('com')

    def test_zone_create_primary_with_all_args(self):
        zone_name = random_zone_name()
        fixture = self.useFixture(ZoneFixture(
            name=zone_name,
            email='primary@example.com',
            description='A primary zone',
            ttl=2345,
            type='PRIMARY',
        ))
        zone = fixture.zone
        self.assertEqual(zone_name, zone.name)
        self.assertEqual('primary@example.com', zone.email)
        self.assertEqual('A primary zone', zone.description)
        self.assertEqual('2345', zone.ttl)
        self.assertEqual('PRIMARY', zone.type)

    def test_zone_create_secondary_with_all_args(self):
        zone_name = random_zone_name()
        fixture = self.useFixture(ZoneFixture(
            name=zone_name,
            description='A secondary zone',
            type='SECONDARY',
            masters='127.0.0.1',
        ))
        zone = fixture.zone
        self.assertEqual(zone_name, zone.name)
        self.assertEqual('A secondary zone', zone.description)
        self.assertEqual('SECONDARY', zone.type)
        self.assertEqual('127.0.0.1', zone.masters)

    def test_zone_set_secondary_masters(self):
        fixture = self.useFixture(ZoneFixture(
            name=random_zone_name(),
            description='A secondary zone',
            type='SECONDARY',
            masters='127.0.0.1',
        ))
        zone = fixture.zone
        self.assertEqual('127.0.0.1', zone.masters)

        zone = self.clients.zone_set(zone.id, masters='127.0.0.2')
        self.assertEqual('127.0.0.2', zone.masters)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient.v2.base import V2Controller


class FloatingIPController(V2Controller):
    def set(self, floatingip_id, ptrdname, description=None, ttl=None):
        data = {
            'ptrdname': ptrdname
        }

        if description is not None:
            data['description'] = description

        if ttl is not None:
            data['ttl'] = ttl

        url = f'/reverse/floatingips/{floatingip_id}'
        return self._patch(url, data=data)

    def list(self, criterion=None):
        url = self.build_url('/reverse/floatingips', criterion)

        return self._get(url, response_key='floatingips')

    def get(self, floatingip_id):
        url = f'/reverse/floatingips/{floatingip_id}'

        return self._get(url)

    def unset(self, floatingip_id):
        data = {'ptrdname': None}

        url = f'/reverse/floatingips/{floatingip_id}'

        return self._patch(url, data=data)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient.v2.base import V2Controller


class QuotasController(V2Controller):
    def list(self, project_id):
        return self._get(f'/quotas/{project_id}')

    def update(self, project_id, values):
        return self._patch(f'/quotas/{project_id}', data=values)

    def reset(self, project_id):
        return self._delete(f'/quotas/{project_id}')



# Copyright 2017 SAP SE
#
# Author: Rudolf Vriend <rudolf.vriend@sap.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from osc_lib.command import command

from designateclient import utils
from designateclient.v2.cli import common
from designateclient.v2.utils import get_all

LOG = logging.getLogger(__name__)


def _format_tsigkey(tsigkey):
    # Remove unneeded fields for display output formatting
    tsigkey.pop('links', None)


class ListTSIGKeysCommand(command.Lister):
    """List tsigkeys"""

    columns = ['id', 'name', 'algorithm', 'secret', 'scope', 'resource_id']

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('--name', help='TSIGKey NAME', required=False)
        parser.add_argument('--algorithm', help='TSIGKey algorithm',
                            required=False)
        parser.add_argument('--scope', help='TSIGKey scope', required=False)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        criterion = {}
        if parsed_args.name is not None:
            criterion['name'] = parsed_args.name
        if parsed_args.algorithm is not None:
            criterion['algorithm'] = parsed_args.algorithm
        if parsed_args.scope is not None:
            criterion['scope'] = parsed_args.scope

        data = get_all(client.tsigkeys.list, criterion)

        cols = self.columns
        return cols, (utils.get_item_properties(s, cols) for s in data)


class ShowTSIGKeyCommand(command.ShowOne):
    """Show tsigkey details"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('id', help='TSIGKey ID')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        data = client.tsigkeys.get(parsed_args.id)
        _format_tsigkey(data)
        return zip(*sorted(data.items()))


class CreateTSIGKeyCommand(command.ShowOne):
    """Create new tsigkey"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('--name', help='TSIGKey Name', required=True)
        parser.add_argument('--algorithm', help='TSIGKey algorithm',
                            required=True)
        parser.add_argument('--secret', help='TSIGKey secret', required=True)
        parser.add_argument('--scope', help='TSIGKey scope', required=True)
        parser.add_argument('--resource-id', help='TSIGKey resource_id',
                            required=True)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        data = client.tsigkeys.create(parsed_args.name, parsed_args.algorithm,
                                      parsed_args.secret, parsed_args.scope,
                                      parsed_args.resource_id)
        _format_tsigkey(data)
        return zip(*sorted(data.items()))


class SetTSIGKeyCommand(command.ShowOne):
    """Set tsigkey properties"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('id', help='TSIGKey ID')
        parser.add_argument('--name', help='TSIGKey Name')
        parser.add_argument('--algorithm', help='TSIGKey algorithm')
        parser.add_argument('--secret', help='TSIGKey secret')
        parser.add_argument('--scope', help='TSIGKey scope')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        data = {}

        if parsed_args.name:
            data['name'] = parsed_args.name
        if parsed_args.algorithm:
            data['algorithm'] = parsed_args.algorithm
        if parsed_args.secret:
            data['secret'] = parsed_args.secret
        if parsed_args.scope:
            data['scope'] = parsed_args.scope

        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.tsigkeys.update(parsed_args.id, data)
        _format_tsigkey(data)
        return zip(*sorted(data.items()))


class DeleteTSIGKeyCommand(command.Command):
    """Delete tsigkey"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument('id', help='TSIGKey ID')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        client.tsigkeys.delete(parsed_args.id)

        LOG.info('TSIGKey %s was deleted', parsed_args.id)



"""
 Copyright 2020 Cloudification GmbH. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from designateclient.functionaltests.base import BaseDesignateTest
from designateclient.functionaltests.client import DesignateCLI
from designateclient.functionaltests.datagen import random_zone_name
from designateclient.functionaltests.v2.fixtures import SharedZoneFixture
from designateclient.functionaltests.v2.fixtures import ZoneFixture


class TestSharedZone(BaseDesignateTest):

    def setUp(self):
        super().setUp()
        self.ensure_tld_exists('com')
        fixture = self.useFixture(ZoneFixture(
            name=random_zone_name(),
            email='test@example.com',
        ))
        self.zone = fixture.zone
        self.target_client = DesignateCLI.as_user('alt')

    def test_list_shared_zones(self):
        shared_zone = self.useFixture(SharedZoneFixture(
            zone_id=self.zone.id,
            target_tenant_id=self.target_client.project_id
        )).zone_share

        shared_zones = self.clients.shared_zone_list(self.zone.id)
        self.assertGreater(len(shared_zones), 0)
        self.assertTrue(self._is_entity_in_list(shared_zone, shared_zones))

    def test_share_and_show_shared_zone(self):
        shared_zone = self.useFixture(SharedZoneFixture(
            zone_id=self.zone.id,
            target_tenant_id=self.target_client.project_id
        )).zone_share

        fetched_shared_zone = self.clients.shared_zone_show(self.zone.id,
                                                            shared_zone.id)

        self.assertEqual(
            shared_zone.created_at, fetched_shared_zone.created_at)
        self.assertEqual(shared_zone.id, fetched_shared_zone.id)
        self.assertEqual(
            shared_zone.project_id, fetched_shared_zone.project_id)
        self.assertEqual(shared_zone.zone_id, fetched_shared_zone.zone_id)

    def test_unshare_zone(self):
        shared_zone = self.useFixture(SharedZoneFixture(
            zone_id=self.zone.id,
            target_tenant_id=self.target_client.project_id
        )).zone_share

        shared_zones = self.clients.shared_zone_list(self.zone.id)
        self.assertTrue(self._is_entity_in_list(shared_zone, shared_zones))

        self.clients.unshare_zone(self.zone.id, shared_zone.id)

        shared_zones = self.clients.shared_zone_list(self.zone.id)
        self.assertFalse(self._is_entity_in_list(shared_zone, shared_zones))



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneauth1 import adapter

from designateclient import exceptions


def get_item_properties(item, fields, mixed_case_fields=(), formatters=None):
    """Return a tuple containing the item properties.

    :param item: a single item resource (e.g. Server, Tenant, etc)
    :param fields: tuple of strings with the desired field names
    :param mixed_case_fields: tuple of field names to preserve case
    :param formatters: dictionary mapping field names to callables
        to format the values
    """
    if formatters is None:
        formatters = {}
    row = []

    for field in fields:
        if field in formatters:
            row.append(formatters[field](item))
        else:
            if field in mixed_case_fields:
                field_name = field.replace(' ', '_')
            else:
                field_name = field.lower().replace(' ', '_')
            if (not hasattr(item, field_name) and
                    (isinstance(item, dict) and field_name in item)):
                data = item[field_name]
            else:
                data = getattr(item, field_name, '')
            if data is None:
                data = ''
            row.append(data)
    return tuple(row)


def get_columns(data):
    """
    Some row's might have variable count of columns, ensure that we have the
    same.

    :param data: Results in [{}, {]}]
    """
    columns = set()

    def _seen(col):
        columns.add(str(col))

    map(lambda item: map(_seen, list(item.keys())), data)
    return list(columns)


def find_resourceid_by_name_or_id(resource_client, name_or_id):
    """Find resource id from its id or name."""
    try:
        # Try to return an uuid
        return str(uuid.UUID(name_or_id))
    except ValueError:
        # Not an uuid => assume it is resource name
        pass

    resources = resource_client.list()
    candidate_ids = [r['id'] for r in resources if r.get('name') == name_or_id]
    if not candidate_ids:
        raise exceptions.ResourceNotFound(
            f'Could not find resource with name "{name_or_id}"')
    elif len(candidate_ids) > 1:
        str_ids = ','.join(candidate_ids)
        raise exceptions.NoUniqueMatch(
            f'Multiple resources with name "{name_or_id}": {str_ids}')
    return candidate_ids[0]


class AdapterWithTimeout(adapter.Adapter):
    """adapter.Adapter wraps around a Session.

    The user can pass a timeout keyword that will apply only to
    the Designate Client, in order:

    - timeout keyword passed to ``request()``
    - timeout keyword passed to ``AdapterWithTimeout()``
    - timeout attribute on keystone session
    """
    def __init__(self, *args, **kw):
        self.timeout = kw.pop('timeout', None)
        super(self.__class__, self).__init__(*args, **kw)

    def request(self, *args, **kwargs):
        if self.timeout is not None:
            kwargs.setdefault('timeout', self.timeout)

        return super().request(*args, **kwargs)



"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import sys
import tempfile
import traceback

import fixtures
from tempest.lib.exceptions import CommandFailed
from testtools.runtest import MultipleExceptions

from designateclient.functionaltests.client import DesignateCLI


class BaseFixture(fixtures.Fixture):

    def __init__(self, user='default', *args, **kwargs):
        """args/kwargs are forwarded to a create method on DesignateCLI"""
        super().__init__()
        self.args = args
        self.kwargs = kwargs
        self.client = DesignateCLI.as_user(user)

    def setUp(self):
        # Sometimes, exceptions are raised in _setUp methods on fixtures.
        # testtools pushes the exception into a MultipleExceptions object along
        # with an artificial SetupError, which produces bad error messages.
        # This just logs those stack traces to stderr for easier debugging.
        try:
            super().setUp()
        except MultipleExceptions as e:
            for i, exc_info in enumerate(e.args):
                print('--- printing MultipleExceptions traceback {} of {} ---'
                      .format(i + 1, len(e.args)), file=sys.stderr)
                traceback.print_exception(*exc_info)
            raise


class ZoneFixture(BaseFixture):
    """See DesignateCLI.zone_create for __init__ args"""

    def _setUp(self):
        super()._setUp()
        self.zone = self.client.zone_create(*self.args, **self.kwargs)
        self.addCleanup(self.cleanup_zone, self.client, self.zone.id)

    @classmethod
    def cleanup_zone(cls, client, zone_id):
        try:
            client.zone_delete(zone_id)
        except CommandFailed:
            pass


class TransferRequestFixture(BaseFixture):
    """See DesignateCLI.zone_transfer_request_create for __init__ args"""

    def __init__(self, zone, user='default', target_user='alt', *args,
                 **kwargs):
        super().__init__(user, *args, **kwargs)
        self.zone = zone
        self.target_client = DesignateCLI.as_user(target_user)

        # the client has a bug such that it requires --target-project-id.
        # when this bug is fixed, please remove this
        self.kwargs['target_project_id'] = self.target_client.project_id

    def _setUp(self):
        super()._setUp()
        self.transfer_request = self.client.zone_transfer_request_create(
            zone_id=self.zone.id,
            *self.args, **self.kwargs
        )
        self.addCleanup(self.cleanup_transfer_request, self.client,
                        self.transfer_request.id)
        self.addCleanup(ZoneFixture.cleanup_zone, self.client, self.zone.id)
        self.addCleanup(ZoneFixture.cleanup_zone, self.target_client,
                        self.zone.id)

    @classmethod
    def cleanup_transfer_request(cls, client, transfer_request_id):
        try:
            client.zone_transfer_request_delete(transfer_request_id)
        except CommandFailed:
            pass


class ExportFixture(BaseFixture):
    """See DesignateCLI.zone_export_create for __init__ args"""

    def __init__(self, zone, user='default', *args, **kwargs):
        super().__init__(user, *args, **kwargs)
        self.zone = zone

    def _setUp(self):
        super()._setUp()
        self.zone_export = self.client.zone_export_create(
            zone_id=self.zone.id,
            *self.args, **self.kwargs
        )
        self.addCleanup(self.cleanup_zone_export, self.client,
                        self.zone_export.id)
        self.addCleanup(ZoneFixture.cleanup_zone, self.client, self.zone.id)

    @classmethod
    def cleanup_zone_export(cls, client, zone_export_id):
        try:
            client.zone_export_delete(zone_export_id)
        except CommandFailed:
            pass


class ImportFixture(BaseFixture):
    """See DesignateCLI.zone_import_create for __init__ args"""

    def __init__(self, zone_file_contents, user='default', *args, **kwargs):
        super().__init__(user, *args, **kwargs)
        self.zone_file_contents = zone_file_contents

    def _setUp(self):
        super()._setUp()

        with tempfile.NamedTemporaryFile() as f:
            f.write(self.zone_file_contents)
            f.flush()

            self.zone_import = self.client.zone_import_create(
                zone_file_path=f.name,
                *self.args, **self.kwargs
            )

        self.addCleanup(self.cleanup_zone_import, self.client,
                        self.zone_import.id)
        self.addCleanup(ZoneFixture.cleanup_zone, self.client,
                        self.zone_import.zone_id)

    @classmethod
    def cleanup_zone_import(cls, client, zone_import_id):
        try:
            client.zone_import_delete(zone_import_id)
        except CommandFailed:
            pass


class RecordsetFixture(BaseFixture):
    """See DesignateCLI.recordset_create for __init__ args"""

    def _setUp(self):
        super()._setUp()
        self.recordset = self.client.recordset_create(
            *self.args, **self.kwargs)
        self.addCleanup(self.cleanup_recordset, self.client,
                        self.recordset.zone_id, self.recordset.id)

    @classmethod
    def cleanup_recordset(cls, client, zone_id, recordset_id):
        try:
            client.recordset_delete(zone_id, recordset_id)
        except CommandFailed:
            pass


class TLDFixture(BaseFixture):
    """See DesignateCLI.tld_create for __init__ args"""

    def __init__(self, user='admin', *args, **kwargs):
        super().__init__(user=user, *args, **kwargs)

    def _setUp(self):
        super()._setUp()
        self.tld = self.client.tld_create(*self.args, **self.kwargs)
        self.addCleanup(self.cleanup_tld, self.client, self.tld.id)

    @classmethod
    def cleanup_tld(cls, client, tld_id):
        try:
            client.tld_delete(tld_id)
        except CommandFailed:
            pass


class TSIGKeyFixture(BaseFixture):
    """See DesignateCLI.tsigkey_create for __init__ args"""

    def __init__(self, user='admin', *args, **kwargs):
        super().__init__(user=user, *args, **kwargs)

    def _setUp(self):
        super()._setUp()
        self.tsigkey = self.client.tsigkey_create(*self.args, **self.kwargs)
        self.addCleanup(self.cleanup_tsigkey(self.client, self.tsigkey.id))

    @classmethod
    def cleanup_tsigkey(cls, client, tsigkey_id):
        try:
            client.tsigkey_delete(tsigkey_id)
        except CommandFailed:
            pass


class BlacklistFixture(BaseFixture):
    """See DesignateCLI.zone_blacklist_create for __init__ args"""

    def __init__(self, user='admin', *args, **kwargs):
        super().__init__(user=user, *args, **kwargs)

    def _setUp(self):
        super()._setUp()
        self.blacklist = self.client.zone_blacklist_create(*self.args,
                                                           **self.kwargs)
        self.addCleanup(self.cleanup_blacklist, self.client, self.blacklist.id)

    @classmethod
    def cleanup_blacklist(cls, client, blacklist_id):
        try:
            client.zone_blacklist_delete(blacklist_id)
        except CommandFailed:
            pass


class SharedZoneFixture(BaseFixture):
    """See DesignateCLI.recordset_create for __init__ args"""

    def __init__(self, zone, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.zone = zone

    def _setUp(self):
        super()._setUp()
        self.zone_share = self.client.zone_share(zone_id=self.zone.id,
                                                 *self.args, **self.kwargs)
        self.addCleanup(self.cleanup_shared_zone, self.client, self.zone.id,
                        self.zone_share.id)

    @classmethod
    def cleanup_shared_zone(cls, client, zone_id, shared_zone_id):
        try:
            client.unshare_zone(zone_id, shared_zone_id)
        except CommandFailed:
            pass



# Copyright 2015 Rackspace Inc.
#
# Author: James Li <james.li@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from designateclient import exceptions
from designateclient.tests import base


class RemoteErrorTestCase(base.TestCase):
    response_dict = {
        'message': None,
        'code': 500,
        'type': None,
        'errors': None,
        'request_id': 1234
    }

    def test_get_error_message(self):
        expected_msg = 'something wrong'
        self.response_dict['message'] = expected_msg
        remote_err = exceptions.RemoteError(**self.response_dict)
        self.assertEqual(expected_msg, remote_err.message)

    def test_get_error_message_with_errors(self):
        expected_msg = "u'nodot.com' is not a 'domainname'"
        errors = {"errors": [
            {"path": ["name"],
             "message": expected_msg,
             "validator": "format",
             "validator_value": "domainname"}
        ]
        }
        self.response_dict['message'] = None
        self.response_dict['errors'] = errors
        remote_err = exceptions.RemoteError(**self.response_dict)
        self.assertEqual(expected_msg, remote_err.message)

    def test_get_error_message_with_type(self):
        expected_msg = 'invalid_object'
        self.response_dict['message'] = None
        self.response_dict['errors'] = None
        self.response_dict['type'] = expected_msg
        remote_err = exceptions.RemoteError(**self.response_dict)
        self.assertEqual(expected_msg, remote_err.message)

    def test_get_error_message_with_unknown_response(self):
        expected_msg = 'invalid_object'
        self.response_dict['message'] = expected_msg
        self.response_dict['unknown'] = 'fake'
        remote_err = exceptions.RemoteError(**self.response_dict)
        self.assertEqual(expected_msg, remote_err.message)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time
import uuid

from designateclient.tests import v2


class TestZones(v2.APIV2TestCase, v2.CrudMixin):
    RESOURCE = 'zones'

    def new_ref(self, **kwargs):
        ref = super().new_ref(**kwargs)
        ref.setdefault("name", uuid.uuid4().hex)
        ref.setdefault("type", "PRIMARY")
        return ref

    def test_create_with_description(self):
        ref = self.new_ref(email="root@example.com", description="Foo")
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.client.zones.create(
            values["name"],
            email=values["email"],
            description=values["description"])
        self.assertRequestBodyIs(json=values)

    def test_create_primary(self):
        ref = self.new_ref(email="root@example.com")
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.client.zones.create(
            values["name"],
            email=values["email"])
        self.assertRequestBodyIs(json=values)

    def test_create_primary_with_ttl(self):
        ref = self.new_ref(email="root@example.com", ttl=60)
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.client.zones.create(
            values["name"],
            email=values["email"],
            ttl=values["ttl"])
        self.assertRequestBodyIs(json=values)

    def test_create_secondary(self):
        ref = self.new_ref(type="SECONDARY", masters=["10.0.0.1"])
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.client.zones.create(
            values["name"],
            type_=values["type"],
            masters=values["masters"])
        self.assertRequestBodyIs(json=values)

    def test_get(self):
        ref = self.new_ref()

        self.stub_entity("GET", entity=ref, id=ref["id"])

        response = self.client.zones.get(ref["id"])
        self.assertEqual(ref, response)

    def test_list(self):
        items = [
            self.new_ref(),
            self.new_ref()
        ]

        self.stub_url("GET", parts=[self.RESOURCE], json={"zones": items})

        listed = self.client.zones.list()
        self.assertList(items, listed)
        self.assertQueryStringIs("")

    def test_update(self):
        ref = self.new_ref()

        self.stub_entity("PATCH", entity=ref, id=ref["id"])

        values = ref.copy()
        del values["id"]

        self.client.zones.update(ref["id"], values)
        self.assertRequestBodyIs(json=values)

    def test_delete(self):
        ref = self.new_ref()

        self.stub_entity("DELETE", id=ref["id"])

        self.client.zones.delete(ref["id"])

        self.assertRequestBodyIs(None)
        self.assertRequestHeaderEqual('X-Designate-Delete-Shares', None)

    def test_delete_with_delete_shares(self):
        ref = self.new_ref()

        self.stub_entity("DELETE", id=ref["id"])

        self.client.zones.delete(ref["id"], delete_shares=True)

        self.assertRequestBodyIs(None)
        self.assertRequestHeaderEqual('X-Designate-Delete-Shares', 'true')

    def test_task_abandon(self):
        ref = self.new_ref()

        parts = [self.RESOURCE, ref["id"], "tasks", "abandon"]
        self.stub_url("POST", parts=parts)

        self.client.zones.abandon(ref["id"])
        self.assertRequestBodyIs(None)

    def test_task_axfr(self):
        ref = self.new_ref()

        parts = [self.RESOURCE, ref["id"], "tasks", "xfr"]
        self.stub_url("POST", parts=parts)

        self.client.zones.axfr(ref["id"])
        self.assertRequestBodyIs(None)

    def test_task_pool_move(self):
        ref = self.new_ref(pool_id=1)

        parts = [self.RESOURCE, ref["id"], "tasks", "pool_move"]
        self.stub_url("POST", parts=parts)

        values = ref.copy()

        self.client.zones.pool_move(ref["id"], values)
        self.assertRequestBodyIs(json=values)


class TestZoneTransfers(v2.APIV2TestCase, v2.CrudMixin):
    def test_create_request(self):
        zone = "098bee04-fe30-4a83-8ccd-e0c496755816"
        project = "123"

        ref = {
            "target_project_id": project
        }

        parts = ["zones", zone, "tasks", "transfer_requests"]
        self.stub_url('POST', parts=parts, json=ref)

        self.client.zone_transfers.create_request(zone, project)
        self.assertRequestBodyIs(json=ref)

    def test_create_request_with_description(self):
        zone = "098bee04-fe30-4a83-8ccd-e0c496755816"
        project = "123"

        ref = {
            "target_project_id": project,
            "description": "My Foo"
        }

        parts = ["zones", zone, "tasks", "transfer_requests"]
        self.stub_url('POST', parts=parts, json=ref)

        self.client.zone_transfers.create_request(
            zone, project, ref["description"])
        self.assertRequestBodyIs(json=ref)

    def test_get_request(self):
        transfer = "098bee04-fe30-4a83-8ccd-e0c496755816"
        project = "098bee04-fe30-4a83-8ccd-e0c496755817"

        ref = {
            "target_project_id": project
        }

        parts = ["zones", "tasks", "transfer_requests", transfer]
        self.stub_url('GET', parts=parts, json=ref)

        self.client.zone_transfers.get_request(transfer)
        self.assertRequestBodyIs("")

    def test_list_request(self):
        project = "098bee04-fe30-4a83-8ccd-e0c496755817"

        ref = [{
            "target_project_id": project
        }]

        parts = ["zones", "tasks", "transfer_requests"]
        self.stub_url('GET', parts=parts, json={"transfer_requests": ref})

        self.client.zone_transfers.list_requests()
        self.assertRequestBodyIs("")

    def test_update_request(self):
        transfer = "098bee04-fe30-4a83-8ccd-e0c496755816"
        project = "098bee04-fe30-4a83-8ccd-e0c496755817"

        ref = {
            "target_project_id": project
        }

        parts = ["zones", "tasks", "transfer_requests", transfer]
        self.stub_url('PATCH', parts=parts, json=ref)

        self.client.zone_transfers.update_request(transfer, ref)
        self.assertRequestBodyIs(json=ref)

    def test_delete_request(self):
        transfer = "098bee04-fe30-4a83-8ccd-e0c496755816"

        parts = ["zones", "tasks", "transfer_requests", transfer]
        self.stub_url('DELETE', parts=parts)

        self.client.zone_transfers.delete_request(transfer)
        self.assertRequestBodyIs("")

    def test_accept_request(self):
        transfer = "098bee04-fe30-4a83-8ccd-e0c496755816"
        key = "foo123"

        ref = {
            "status": "COMPLETE"
        }

        parts = ["zones", "tasks", "transfer_accepts"]
        self.stub_url('POST', parts=parts, json=ref)

        request = {
            "key": key,
            "zone_transfer_request_id": transfer
        }
        self.client.zone_transfers.accept_request(transfer, key)
        self.assertRequestBodyIs(json=request)

    def test_get_accept(self):
        accept_id = "098bee04-fe30-4a83-8ccd-e0c496755816"

        ref = {
            "status": "COMPLETE"
        }

        parts = ["zones", "tasks", "transfer_accepts", accept_id]
        self.stub_url('GET', parts=parts, json=ref)

        response = self.client.zone_transfers.get_accept(accept_id)
        self.assertEqual(ref, response)

    def test_list_accepts(self):
        accept_id = "098bee04-fe30-4a83-8ccd-e0c496755816"

        ref = {
            "id": accept_id,
            "status": "COMPLETE"
        }

        parts = ["zones", "tasks", "transfer_accepts"]
        self.stub_url('GET', parts=parts, json={"transfer_accepts": ref})

        self.client.zone_transfers.list_accepts()
        self.assertRequestBodyIs("")


class TestZoneExports(v2.APIV2TestCase, v2.CrudMixin):
    def new_ref(self, **kwargs):
        ref = super().new_ref(**kwargs)
        ref.setdefault("zone_id", uuid.uuid4().hex)
        ref.setdefault("created_at", time.strftime("%c"))
        ref.setdefault("updated_at", time.strftime("%c"))
        ref.setdefault("status", 'PENDING')
        ref.setdefault("version", '1')
        return ref

    def test_create_export(self):
        zone = uuid.uuid4().hex
        ref = {}

        parts = ["zones", zone, "tasks", "export"]
        self.stub_url('POST', parts=parts, json=ref)

        self.client.zone_exports.create(zone)
        self.assertRequestBodyIs(json=ref)

    def test_get_export(self):
        ref = self.new_ref()

        parts = ["zones", "tasks", "exports", ref["id"]]
        self.stub_url('GET', parts=parts, json=ref)
        self.stub_entity("GET", parts=parts, entity=ref, id=ref["id"])

        response = self.client.zone_exports.get_export_record(ref["id"])
        self.assertEqual(ref, response)

    def test_list_exports(self):
        items = [
            self.new_ref(),
            self.new_ref()
        ]

        parts = ["zones", "tasks", "exports"]
        self.stub_url('GET', parts=parts, json={"exports": items})

        listed = self.client.zone_exports.list()
        self.assertList(items, listed["exports"])
        self.assertQueryStringIs("")

    def test_delete_export(self):
        ref = self.new_ref()

        parts = ["zones", "tasks", "exports", ref["id"]]
        self.stub_url('DELETE', parts=parts, json=ref)
        self.stub_entity("DELETE", parts=parts, id=ref["id"])

        self.client.zone_exports.delete(ref["id"])
        self.assertRequestBodyIs(None)

    def test_get_export_file(self):
        ref = self.new_ref()

        parts = ["zones", "tasks", "exports", ref["id"], "export"]
        self.stub_url('GET', parts=parts, json=ref)
        self.stub_entity("GET", parts=parts, entity=ref, id=ref["id"])

        response = self.client.zone_exports.get_export(ref["id"])
        self.assertEqual(ref, response)


class TestZoneImports(v2.APIV2TestCase, v2.CrudMixin):
    def new_ref(self, **kwargs):
        ref = super().new_ref(**kwargs)
        ref.setdefault("zone_id", uuid.uuid4().hex)
        ref.setdefault("created_at", time.strftime("%c"))
        ref.setdefault("updated_at", time.strftime("%c"))
        ref.setdefault("status", 'PENDING')
        ref.setdefault("message", 'Importing...')
        ref.setdefault("version", '1')
        return ref

    def test_create_import(self):
        zonefile = '$ORIGIN example.com'

        parts = ["zones", "tasks", "imports"]
        self.stub_url('POST', parts=parts, json=zonefile)

        self.client.zone_imports.create(zonefile)
        self.assertRequestBodyIs(body=zonefile)

    def test_get_import(self):
        ref = self.new_ref()

        parts = ["zones", "tasks", "imports", ref["id"]]
        self.stub_url('GET', parts=parts, json=ref)
        self.stub_entity("GET", parts=parts, entity=ref, id=ref["id"])

        response = self.client.zone_imports.get_import_record(ref["id"])
        self.assertEqual(ref, response)

    def test_list_imports(self):
        items = [
            self.new_ref(),
            self.new_ref()
        ]

        parts = ["zones", "tasks", "imports"]
        self.stub_url('GET', parts=parts, json={"imports": items})

        listed = self.client.zone_imports.list()
        self.assertList(items, listed["imports"])
        self.assertQueryStringIs("")

    def test_delete_import(self):
        ref = self.new_ref()

        parts = ["zones", "tasks", "imports", ref["id"]]
        self.stub_url('DELETE', parts=parts, json=ref)
        self.stub_entity("DELETE", parts=parts, id=ref["id"])

        self.client.zone_imports.delete(ref["id"])
        self.assertRequestBodyIs(None)


class TestZoneShared(v2.APIV2TestCase, v2.CrudMixin):
    def setUp(self):
        super().setUp()
        self.zone_id = str(uuid.uuid4())
        self.target_project_id = str(uuid.uuid4())
        self.project_id = str(uuid.uuid4())
        self.created_at = time.strftime("%c")
        self.updated_at = time.strftime("%c")

    def new_ref(self, **kwargs):
        ref = super().new_ref(**kwargs)
        ref.setdefault("zone_id", self.zone_id)
        ref.setdefault("target_project_id", self.target_project_id)
        ref.setdefault("project_id", self.project_id)
        ref.setdefault("created_at", self.created_at)
        ref.setdefault("updated_at", self.updated_at)
        return ref

    def test_share_a_zone(self):
        json_body = {"target_project_id": self.target_project_id}

        expected = self.new_ref()

        self.stub_entity('POST', parts=['zones', self.zone_id, 'shares'],
                         entity=expected, json=json_body)

        response = self.client.zone_share.create(self.zone_id,
                                                 self.target_project_id)

        self.assertRequestBodyIs(json=json_body)
        self.assertEqual(expected, response)

    def test_get_zone_share(self):
        expected = self.new_ref()

        parts = ["zones", self.zone_id, "shares"]
        self.stub_entity("GET", parts=parts, entity=expected,
                         id=expected["id"])

        response = self.client.zone_share.get(self.zone_id, expected["id"])

        self.assertRequestBodyIs(None)
        self.assertEqual(expected, response)

    def test_list_zone_shares(self):
        items = [
            self.new_ref(),
            self.new_ref()
        ]

        parts = ["zones", self.zone_id, "shares"]
        self.stub_entity('GET', parts=parts, entity={"shared_zones": items})

        listed = self.client.zone_share.list(self.zone_id)

        self.assertList(items, listed)
        self.assertQueryStringIs("")

    def test_delete_zone_share(self):
        ref = self.new_ref()

        parts = ["zones", self.zone_id, "shares", ref["id"]]
        self.stub_url('DELETE', parts=parts)

        response = self.client.zone_share.delete(self.zone_id, ref["id"])

        self.assertRequestBodyIs(None)
        self.assertEqual('', response)



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient.v2 import base


class ServiceStatusesController(base.V2Controller):
    def list(self, criterion=None, marker=None, limit=None):
        url = self.build_url('/service_statuses', criterion, marker, limit)

        return self._get(url, response_key='service_statuses')

    def get(self, service_status_id):
        url = f'/service_statuses/{service_status_id}'

        return self._get(url)



"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from tempest.lib.cli import output_parser


class Model:

    def __str__(self):
        return str(self.__dict__)


class FieldValueModel(Model):
    """This converts cli output from messy lists/dicts to neat attributes."""

    def __init__(self, out):
        """This parses output with fields and values like:

            +----------------+------------------------------+
            | Field          | Value                        |
            +----------------+------------------------------+
            | action         | CREATE                       |
            | created_at     | 2015-08-20T17:22:17.000000   |
            | description    | None                         |
            +----------------+------------------------------+

        These are then accessible as:

            model.action
            model.created_at
            model.description

        """
        table = output_parser.table(out)

        # Because the output_parser handles Values with multiple lines
        # in additional Field/Value pairs with Field name '', the following
        # code is necessary to aggregate Values.
        #
        # The list of Field/Value pairs is in-order, so we can append Value
        # continuation to the previously seen Field, with a newline separator.
        value_lines = []
        prev_field = None
        for field, value in table['values']:
            if field == '':
                value_lines.append(value)
                setattr(self, prev_field, '\n'.join(value_lines))
            else:
                setattr(self, field, value)
                prev_field = field
                value_lines = [value]


class ListEntryModel(Model):

    def __init__(self, fields, values):
        for k, v in zip(fields, values):
            setattr(self, k, v)


class ListModel(Model, list):

    def __init__(self, out):
        """This parses an output table with any number of headers, and any
        number of entries:

            +--------------------------------------+----------+---------+
            | id                                   | name     | type    |
            +--------------------------------------+----------+---------+
            | e658a875-1024-4f88-a347-e5b244ec5a10 | aaa.com. | PRIMARY |
            +--------------------------------------+----------+---------+
            | 98d1fb5f-2954-448e-988e-6f1df0f24c52 | bbb.com. | PRIMARY |
            +--------------------------------------+----------+---------+

        These are then accessible as:

            model[0].name == 'aaa.com.'
            model[1].name == 'bbb.com.'

        """
        table = output_parser.table(out)
        for entry in table['values']:
            self.append(ListEntryModel(table['headers'], entry))



# Copyright (c) 2015 Thales Services SAS
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
import uuid

from designateclient import exceptions
from designateclient.tests import base
from designateclient import utils


LIST_MOCK_RESPONSE = [
    {'id': '13579bdf-0000-0000-abcd-000000000001', 'name': 'abcd'},
    {'id': '13579bdf-0000-0000-baba-000000000001', 'name': 'baba'},
    {'id': '13579bdf-0000-0000-baba-000000000002', 'name': 'baba'},
]


class UtilsTestCase(base.TestCase):

    def _find_resourceid_by_name_or_id(self, name_or_id, by_name=False):
        resource_client = mock.Mock()
        resource_client.list.return_value = LIST_MOCK_RESPONSE
        resourceid = utils.find_resourceid_by_name_or_id(
            resource_client, name_or_id)
        self.assertEqual(by_name, resource_client.list.called)
        return resourceid

    def test_find_resourceid_with_hyphen_uuid(self):
        expected = str(uuid.uuid4())
        observed = self._find_resourceid_by_name_or_id(expected)
        self.assertEqual(expected, observed)

    def test_find_resourceid_with_nonhyphen_uuid(self):
        expected = str(uuid.uuid4())
        fakeid = expected.replace('-', '')
        observed = self._find_resourceid_by_name_or_id(fakeid)
        self.assertEqual(expected, observed)

    def test_find_resourceid_with_unique_resource(self):
        observed = self._find_resourceid_by_name_or_id('abcd', by_name=True)
        self.assertEqual('13579bdf-0000-0000-abcd-000000000001', observed)

    def test_find_resourceid_with_nonexistent_resource(self):
        self.assertRaises(exceptions.ResourceNotFound,
                          self._find_resourceid_by_name_or_id,
                          'taz', by_name=True)

    def test_find_resourceid_with_multiple_resources(self):
        self.assertRaises(exceptions.NoUniqueMatch,
                          self._find_resourceid_by_name_or_id,
                          'baba', by_name=True)
